@@ -4,10 +4,7 @@
 #'
 #' Creates various types of plots to visualize and evaluate the trained SOM model.
 #'
-#' @import RColorBrewer aweSOM dplyr kohonen maptree
-#' @importFrom grDevices colorRampPalette
-#' @importFrom stats cutree dist hclust na.omit
-#' @importFrom utils read.csv setTxtProgressBar txtProgressBar
+#' @import kohonen
 #' @param model A trained SOM model object.
 #' @param plot_type An integer specifying the type of plot to generate. Options are:
 #'   \describe{
@@ -36,33 +33,45 @@
 
 generatePlot <- function(model, plot_type, data = NULL) {
 
-  # Select plot based on plot_type
-  if (plot_type == 1) {
-    # Training progress for SOM.
-    plot(model, type="changes")
-
-  } else if (plot_type == 2) {
-    # Node count plot (for map quality).
-    plot(model, type="count", main="Node Counts")
-
-  } else if (plot_type == 3) {
-    # U-matrix visualization (similarity between each node and its neighbors).
-    plot(model, type="dist.neighbours", main = "SOM neighbour distances")
-
-  } else if (plot_type == 4) {
-    # Weight vector view (patterns in the distribution of samples and variables).
-    plot(model, type="codes")
-
-  } else if (plot_type == 5) {
-    # Kohonen heatmap creation (distribution of single variables across the map);
-    # if there are improper column names, it could potentially result in an error.
-    for (i in 1:ncol(data)){
-      plot(model, type = "property", property = getCodes(model)[,i], main=colnames(getCodes(model))[i])
-    }
-  } else {
-    stop("Invalid plot_type.")
+  if (!inherits(model, "kohonen")) {
+    stop("`model` must be a trained SOM object from the kohonen package.")
   }
 
-  # Print the plot
-  print(plot)
+  # Select plot based on plot_type
+  if (plot_type == 1) {
+    plot(model, type = "changes")
+
+  } else if (plot_type == 2) {
+    plot(model, type = "count", main = "Node Counts")
+
+  } else if (plot_type == 3) {
+    plot(model, type = "dist.neighbours", main = "SOM Neighbour Distances")
+
+  } else if (plot_type == 4) {
+    plot(model, type = "codes", main = "Weight Vectors")
+
+  } else if (plot_type == 5) {
+
+    if (is.null(data)) {
+      stop("For plot_type = 5, `data` must be provided.")
+    }
+
+    codes <- getCodes(model)
+
+    if (ncol(data) != ncol(codes)) {
+      stop("Number of columns in `data` must match the SOM model variables.")
+    }
+
+    var_names <- colnames(codes)
+    if (is.null(var_names)) {
+      var_names <- paste0("Var_", seq_len(ncol(codes)))
+    }
+
+    for (i in seq_len(ncol(codes))) {
+      plot(model, type = "property", property = codes[, i], main = var_names[i])
+    }
+
+  } else {
+    stop("Invalid `plot_type`. Must be an integer between 1 and 5.")
+  }
 }
